"""Version information for LlamaAgent.

Author: Nik Jois <nikjois@llamasearch.ai>
"""

__version__ = "1.1.4"
__version_info__ = (1, 1, 4)

# Version components
MAJOR = 1
MINOR = 1
PATCH = 4

# Build metadata
BUILD_DATE = "2025-08-09"
BUILD_COMMIT = "current"
BUILD_BRANCH = "main"

# Compatibility
version = __version__
version_info = __version_info__
